import glob
import os
import random
import shutil
import tempfile
from os.path import abspath, basename, splitext

from importlib_resources import files

import sltxpkg.config as sc
import sltxpkg.data.recipes
import sltxpkg.globals as sg
import sltxpkg.lithie.compile.cooking.recipe_exceptions as rex
import sltxpkg.lithie.compile.tools as tools
import sltxpkg.util as su
from sltxpkg.globals import print_idx
from sltxpkg.util import create_multiple_replacer

BRACE_REPLACER = create_multiple_replacer({
    ':braceo:': '{',
    ':bracee:': '}',
})


class Recipe():
    settings = {
        'name': '',
        'author': '',
        'hooks': {
            'pre': [],
            'in': [],  # before retrieval
            'post': []  # before cleanup
        },
        # cleanup will be determined by global settings
        'cleanup_cmds': [],
        'args': '',
        'extra_args': [],
        'tools': [],
        # to retrieve files:
        'wanted_files': [],
        'executable': '',
        'run': [],
        'quiet': ''
    }

    quiet = ''

    def __init__(self, recipe_path: str, file: str, idx: str):
        super().__init__()
        self.file = file
        self.idx = idx
        recipe_full_path = recipe_path
        if not os.path.isfile(recipe_full_path):
            recipe_full_path = str(
                files(sltxpkg.data.recipes).joinpath(recipe_path))
        if not os.path.isfile(recipe_full_path):
            print_idx(self.idx, "Recipe " + recipe_full_path +
                      " was not found. Exiting.")
            exit(1)
        print_idx(self.idx, "Loading recipe: " + recipe_full_path)
        y_conf = su.load_yaml(recipe_full_path)
        self.settings = {**self.settings, **y_conf}
        if sg.args.quiet:
            self.quiet = self.settings['quiet']
        self.__process_tools()
        self.__sanitize_extra_args()  # we need them as a single string

    def __process_tools(self):
        for tool in self.settings['tools']:
            getattr(tools, 'tool_' + tool)(self)

    def __sanitize_extra_args(self):
        self.settings['extra_args'].extend(sg.args.extra_arguments)
        self.settings['extra_args'] = " ".join(self.settings['extra_args'])

    @staticmethod
    def get_default_recipes() -> [str]:
        return [f for f in os.listdir() if f.endswith(".recipe")]

    # format; TODO: maybe cache the results
    def __f(self, t: str) -> str:
        for _ in range(sg.configuration[sg.C_FORMAT_MAX]):
            t = t.format(**self.settings, **sg.configuration, file=self.file,
                         filenoext=splitext(self.file)[0], do_quiet=self.quiet,
                         file_base_noext=splitext(basename(self.file))[0],
                         tmp=tempfile.gettempdir(),
                         out_dir=os.path.join("{cache_dir}", su.sanitize_filename(abspath(self.file))))
        return BRACE_REPLACER(t)

    def __runcmds(self, cmds: [str]):
        for cmd in cmds:
            cmd = self.__f(cmd)  # expand
            if sg.args.verbose:
                print_idx(self.idx, "  - " + cmd)
            os.system(cmd)

    def __runhooks(self, hookid: str):
        if sg.args.verbose:
            print_idx(self.idx, "> Hooks for \"" + hookid + "\"")
        self.__runcmds(self.settings['hooks'][hookid])

    def __critical_abort(self, code: int):
        print_idx(self.idx, "Collecting files in working directory")
        archive = shutil.make_archive(os.path.join(
            os.getcwd(), 'sltx-log-' + su.get_now() + '-' + su.sanitize_filename(self.file)), 'zip',
            self.__f("{out_dir}"))
        print_idx(self.idx, "  - Created: \"" + archive +
                  "\" (" + os.path.basename(archive) + ")")

        # We have to force latexmk into think it has to re-run
        # TODO: We should check if not aux but we do this
        # We do this as we need a change and the percent itself gets consumed
        # This ensures a different sequence every time that will be deleted
        # on a successful run
        with open(self.__f("{out_dir}/{file_base_noext}.aux"), 'a') as f:
            f.write('%% sltx errormark' + str(random.random()) +
                    " - " + str(random.random()))
        # automatic analyze
        os.system('sltx analyze "' + archive + '"')
        raise rex.RecipeException(archive,
                                  'Recipe for ' + str(self.idx) + ' failed with code: ' + str(
                                      code) + '. See logfile: \"' + archive + "\"")

    def __save_files(self, our_dir: str):
        """Retrieves the resulting files by patterns

        Args:
            our_dir (str): the path to the target directory for caught files
        """
        print_idx(self.idx, '> Retrieving resulting files to "' + our_dir + '"')
        got_files = []
        for wf in self.settings['wanted_files'] + sg.configuration[sg.C_WANTED_FILES]:
            wf = self.__f(wf)
            if sg.args.verbose:
                print_idx(
                    self.idx, '  - Retrieving files for pattern "' + wf + '"')
            wanted = glob.glob(os.path.join(
                sg.configuration[sg.C_WORKING_DIR], wf))
            for f in wanted:
                if sg.args.verbose:
                    print_idx(self.idx, "Saving \"" + f + "\" ")
                shutil.copy2(f, our_dir)
            got_files += wanted

        if not sg.args.verbose:
            print_idx(self.idx, "Saved files (" + str(got_files) + ")")

    def run(self):
        """Executes the configured Recipe
        """
        print_idx(self.idx, "Processing file: " + self.file, pre='\n')
        sc.assure_dirs()  # Ensure Working diSr and texmf home
        sc.assure_dir('file cache', self.__f("{out_dir}"), True)
        print_idx(self.idx, self.__f(
            "> Running recipe \"{name}\" by \"{author}\"."))
        self.__runhooks('pre')

        print_idx(self.idx, "> Running the compile commands (" +
                  str(len(self.settings['run'])) + ")")
        for i, cmd in enumerate(self.settings['run']):
            cmd = self.__f(cmd)  # expand
            if sg.args.verbose:
                print_idx(self.idx, "  - " + cmd)
            fback = os.system(cmd)
            if fback != 0:
                print_idx(self.idx,
                          "\033[31m  ! The command failed. Initiating critical abort...\033[m")
                self.__critical_abort(fback)

        self.__runhooks('in')
        self.__save_files(os.getcwd())
        self.__runhooks('post')
        if sg.configuration[sg.C_CLEANUP]:
            print_idx(self.idx, "> Cleaning up (configured by configuration)")
            self.__runcmds(self.settings['cleanup_cmds'])



import glob
import os
import re
import shutil
import sys
from concurrent import futures
from pathlib import Path
from subprocess import PIPE, Popen  # execution

import sltxpkg.globals as sg
import sltxpkg.util as su
from sltxpkg.config import load_dependencies_config, write_to_log
from sltxpkg.globals import (C_AUTODETECT_DRIVERS, C_CLEANUP, C_DOWNLOAD_DIR, C_DRIVER_PATTERNS,
                             C_DRIVERS, C_RECURSIVE, print_idx)
from sltxpkg.log_control import LOGGER

loaded = []


def detect_driver(idx: str, url: str) -> str:
    """Tries to match the given patterns to [description]
    This could be optimized by pre-compile the given patterns.

    Args:
        idx (str): the current index number for logging
        url (str): the url to adapt the driver from

    Returns:
        (str): The driver key to use
    """
    print_idx(idx, " - Auto-detecting driver...")
    for key, patterns in sg.configuration[C_DRIVER_PATTERNS].items():
        for pattern in patterns:
            if re.search(pattern, url):
                return key
    print_idx(idx, " ! No driver found...")
    sys.exit(1)


def split_grab_pattern(pattern: str, default_target: str) -> (str, str):
    """Performes splits on grab patterns ("source=>target")
       will fill in the given default target, if split does not present one

    Args:
        pattern (str): The pattern to split
        default_target (str): The target to be auto-filled if none given

    Returns:
        (str,str): pattern and pair
    """
    parts = pattern.split('=>', 1)
    target = default_target if len(parts) == 1 else parts[1]
    return parts[0], target


class DependencyProfileException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


def extend_grab_from_local(idx: str, driver_target_dir: str, data: dict) -> (list, list):
    """May install extra profiles

       This method will check for a local dep file, and if present
       check for a profiles key. if present it will check for a selected profile in the data dict.
       If so, select it, if none, set the default
    Args:
        idx (str): index to use in multithreading
        driver_target_dir (str): target dir of the current driver
        data (str): data dict for local configuration
    Returns:
        (list,list) - a list of additional dependencies in the format 'files, folder'
    """
    if 'dep' not in data:
        dep = sg.DEFAULT_DEPENDENCY
    else:
        dep = data['dep']
    dep_files = glob.glob(os.path.join(driver_target_dir, dep), recursive=True)
    if len(dep_files) <= 0:
        return [], []

    # load file and check for default
    # TODO: avoid reloading if recursive?
    file_profiles = {}
    for dep_file in dep_files:
        file_profiles = load_dependencies_config(dep_file, file_profiles)

    if 'profiles' not in file_profiles:
        return [], []

    file_profiles = file_profiles['profiles']
    # Note: I do want always a default profile so it makes live easier for me
    if 'default' not in file_profiles:
        raise DependencyProfileException('No default profile for enlisted profiles. Found: ' + str(file_profiles))
    if 'profile' in data:
        requested_profile = data['profile']
        if requested_profile not in file_profiles:
            raise DependencyProfileException('Requested profile (' + requested_profile + ') not found. Found: ' +
                                             str(file_profiles))
    else:
        requested_profile = 'default'
    added_profiles: dict = file_profiles[requested_profile]
    print_idx(idx, ' > Loaded profile (' + requested_profile + '): ' + str(added_profiles))
    # TODO: this may be beautified
    grab_files = added_profiles['grab-files'] if 'grab-files' in added_profiles else ""
    grab_dirs = added_profiles['grab_dirs'] if 'grab_dirs' in added_profiles else ""
    return grab_files, grab_dirs


def grab_from(idx: str, path: str, data: dict, target: str, key: str, grabber, extras: list) -> bool:

    if key not in data:
        if len(extras) == 0:
            print_idx(idx, " ! Key '" + key + "' not found. Won't grab any...")
            return False
        else:
            data[key] = extras
    else:
        data[key].extend(extras)

    grabs = []
    for grab_pattern in set(data[key]):
        cur_grab_pattern = split_grab_pattern(grab_pattern, target)
        # maybe forbid level up?
        grabs.extend(map(lambda x, pattern=cur_grab_pattern: (x, pattern[1]),
                         glob.glob(os.path.join(path, cur_grab_pattern[0]), recursive=True)))

    # extra so i can setup installer afterwards more easily
    print_idx(idx, " > Grabbing the following for installation: "
              + str([os.path.relpath(f[0], path) for f in grabs]))
    for grab in grabs:
        grabber(grab, target, path)
    return True


def f_grab_files(data: (str, str), target: str, path: str):
    file_target = os.path.join(target, data[1]) if data[1] != target else os.path.join(
        data[1], os.path.relpath(data[0], path))
    Path(file_target).parent.mkdir(parents=True, exist_ok=True)
    shutil.copy2(data[0], file_target)


def f_grab_dirs(data: (str, str), target: str, path: str):
    # only choose relative path
    dir_target = os.path.join(target, data[1]) if data[1] != target else os.path.join(
        data[1], os.path.relpath(data[0], path))
    Path(dir_target).parent.mkdir(parents=True, exist_ok=True)
    if sys.version_info >= (3, 8, 0):  # we have exist is ok
        shutil.copytree(data[0], dir_target, dirs_exist_ok=True)
    else:
        LOGGER.info("Python version below 3.8, falling back with distutils!")
        import distutils.dir_util as du
        du.copy_tree(data[0], dir_target)


def write_proc_to_log(idx: str, stream, mirror: bool):
    while True:
        line = stream.readline()
        if not line:
            break
        line_utf8 = line.decode('utf-8')
        write_to_log(line_utf8)
        if mirror:
            print_idx(idx, line_utf8)


def grab_stuff(idx: str, dep_name: str, target_dir: str, data: dict, target: str):

    extra_files, extra_dirs = extend_grab_from_local(idx, target_dir, data)

    print_idx(idx, " > Grabbing dependencies for " + dep_name)
    print_idx(idx, "   - Grabby-Grab-Grab files from \"" + target_dir + "\"...")
    got_files = grab_from(idx, target_dir, data, target,
                          'grab-files', f_grab_files, extra_files)
    print_idx(idx, " - Grabby-Grab-Grab dirs from \"" + target_dir + "\"...")
    got_dirs = grab_from(idx, target_dir, data, target,
                         'grab-dirs', f_grab_dirs, extra_dirs)
    if not got_files and not got_dirs:
        print_idx(idx, " ! No grabs performed!")
        write_to_log("No grabs performed for: " + dep_name)


def get_target_dir(data: dict, dep_name: str, driver: str):
    return sg.configuration[C_DRIVERS][driver]["target-dir"].format(
        **data, **sg.configuration, dep_name=dep_name)


def recursive_dependencies(idx: str, driver_target_dir: str, data: dict, dep_name: str, target: str):
    if 'dep' not in data:
        print_idx(idx, "No 'dep' key found for dep: " + dep_name +
                  " using the default (" + sg.DEFAULT_DEPENDENCY + ")")
        data['dep'] = sg.DEFAULT_DEPENDENCY
    dep_files = glob.glob(os.path.join(
        driver_target_dir, data['dep']), recursive=True)
    print_idx(idx, " - Found dep-config: " + str(dep_files))

    if len(dep_files) <= 0:
        return

    new_dependencies = {}
    for dep_file in dep_files:
        new_dependencies = load_dependencies_config(dep_file, new_dependencies)

    _install_dependencies(idx, new_dependencies, target)


def use_driver(idx: str, data: dict, dep_name: str, driver: str, target: str):
    # default no arguments
    if "args" not in data:
        data["args"] = ""
    driver_data = sg.configuration[C_DRIVERS][driver]
    command = driver_data["command"].format(**data, **sg.configuration, dep_name=dep_name)
    driver_target_dir = get_target_dir(data, dep_name, driver)
    if driver_data["needs-delete"] and os.path.isdir(driver_target_dir):
        print_idx(idx, " - Target folder " + driver_target_dir +
                  "exists. Will be deleted as the driver needs this")
        shutil.rmtree(driver_target_dir)

    if driver_data["needs-create"] and not os.path.isdir(driver_target_dir):
        print_idx(idx, " - Target folder " +
                  driver_target_dir + " needs creation")
        os.makedirs(driver_target_dir)

    print_idx(idx, " > Executing: " + command)
    with Popen(command, stdout=PIPE, stderr=PIPE, shell=True) as feedback:
        return_code = feedback.wait()
        write_proc_to_log(idx, feedback.stdout, False)
        if return_code != 0:
            print_idx(idx, " - Error-Log of Driver:")
        write_proc_to_log(idx, feedback.stderr, return_code != 0)

    if sg.configuration[C_RECURSIVE]:
        recursive_dependencies(idx, driver_target_dir, data, dep_name, target)

    if return_code != 0:
        print_idx(idx, " ! Driver failed with code" +
                  str(feedback) + "exiting.")
        sys.exit(return_code)

    grab_stuff(idx, dep_name, driver_target_dir, data, target)


def install_dependency(name: str, idx: str, data: dict, target: str):
    print_idx(idx, 'Loading "' + name + '"')

    if "url" not in data:
        print_idx(idx, " ! The dependency did not have an url-tag attached")
        exit(1)

    url = data["url"]
    print_idx(idx, ' - Loading from: "' + url + '"')
    if "driver" not in data:
        if not sg.configuration[C_AUTODETECT_DRIVERS]:
            print_idx(idx, " ! No driver given and auto-detection disabled!")
        else:
            data["driver"] = detect_driver(idx, url)

    driver = data["driver"]
    print_idx(idx, " - Using driver: \"" + driver + "\"")

    if name in loaded:
        print_idx(idx, " > Skipping retrieval " + name +
                  " as it was already loaded by another dep.")
        grab_stuff(idx, name, get_target_dir(data, name, driver), data, target)
        return

    loaded.append(name)

    if driver not in sg.configuration[C_DRIVERS]:
        print_idx(idx, ' ! The selected driver is unknown. Loaded:' +
                  sg.configuration[C_DRIVERS])
        sys.exit(2)
    use_driver(idx, data, name, driver, target)


def _finish_runners(runners: list):
    futures.wait(runners)
    for runner in runners:
        if runner.result() is not None:
            LOGGER.info(runner.result())


def _install_dependencies(idx: str, dep_dict: dict, target: str, first: bool = False):
    """Will install dependencies in an multi-threaded environment and may be called recursively

    Args:
        idx (str): The index to be run in (will start with 0)
        dep_dict (dict): Dependencies to fetch with this run
        target (str): The target directory for the fetch
        first (bool, optional): Flag for the initial run. Defaults to False.
    """
    if 'dependencies' not in dep_dict:
        return

    with futures.ThreadPoolExecutor(max_workers=sg.args.threads) as pool:
        runners = []
        for i, dep in enumerate(dep_dict['dependencies']):
            runners.append(pool.submit(install_dependency, dep, str(i) if first else idx + "." + str(i),
                                       dep_dict['dependencies'][dep], target))
        _finish_runners(runners)


def _install_dependencies_guard():
    """Cheap command line guard which will check for valid keys
    """
    if "target" not in sg.dependencies or "dependencies" not in sg.dependencies:
        LOGGER.error(
            "The dependency-file must supply a 'target' and an 'dependencies' key!")
        sys.exit(1)


def _install_dependencies_cleanup():
    """This will be run after the requested dependencies have been installed.
    """
    if sg.configuration[C_CLEANUP]:
        LOGGER.info("> Cleaning up the download directory, as set.")
        shutil.rmtree(sg.configuration[C_DOWNLOAD_DIR])

    LOGGER.info("Loaded: " + str(loaded))
    if not sg.configuration[C_RECURSIVE]:
        LOGGER.info("Recursion was disabled.")

    LOGGER.info("Dependency installation for %s completed.",
                sg.dependencies["target"])


def install_dependencies(target: str = su.get_sltx_tex_home()) -> None:
    """Download and unpack given dependencies to the given target directory

    Args:
        target (str, optional): The target folder. Defaults to su.get_sltx_tex_home().
    """
    _install_dependencies_guard()

    write_to_log("====Dependencies for:" + sg.dependencies["target"] + "\n")
    LOGGER.info("\nDependencies for: " + sg.dependencies["target"])
    LOGGER.info("Installing to: %s\n", target)

    _install_dependencies('0', sg.dependencies, target, first=True)
    _install_dependencies_cleanup()



import os  # list directory
import sys  # cmd line args

import sltxpkg.config as sc
from sltxpkg import globals as sg
from sltxpkg.cmd_line_args import parser, sub_parser
from sltxpkg.globals import DEFAULT_CONFIG, LOCAL_CONFIG
from sltxpkg.log_control import LOGGER, log_set_file_handler


def autoload_config(path: str, name: str):
    if os.path.isfile(path):
        LOGGER.info("Auto-load %s-config: '%s'", name, path)
        sc.load_configuration(path)


def retrieve_by_alias(tc: str):
    return [alias[0] for alias in sub_parser.cmds.items() if tc in alias[1][0][1]][0]


def _file_guard():
    if hasattr(sg.args, 'files') and not sg.args.files:
        if sg.args.verbose:
            LOGGER.info("Set default files to: " +
                        str(sg.configuration[sg.C_DEFAULT_FILES]))
        sg.args.files = sg.configuration[sg.C_DEFAULT_FILES]
        if not sg.args.files:
            LOGGER.error("No files supplied")
            exit(1)


def run(args=None):
    if args is None:
        args = sys.argv[1:]
    if len(args) < 1:
        parser.parse_args(['-h'])
    sg.args = parser.parse_args(args)

    if sg.args.log:
        log_set_file_handler()

    if not sg.args.command:
        parser.parse_args(['-h'])

    autoload_config(DEFAULT_CONFIG, 'default')
    autoload_config(LOCAL_CONFIG, 'local')

    _file_guard()

    if sg.args.threads < 0:
        if sg.args.verbose:
            LOGGER.info("Set default thread-count to: %d",
                        sg.configuration[sg.C_DEFAULT_THREADS])
        sg.args.threads = sg.configuration[sg.C_DEFAULT_THREADS]

    if sg.args.config is not None:
        sc.load_configuration(sg.args.config)

    cmd = None
    try:
        tc = sg.args.command.lower()
        cmd = sub_parser.cmds[tc if tc in sub_parser.cmds else retrieve_by_alias(
            tc)]
    except KeyError:
        LOGGER.error("The supplied command: %s is unknown. Choose one of: %s",
                     sg.args.command, list(sub_parser.cmds.keys()))
        exit(1)

    cmd[0][0]()



import argparse
import os

import sltxpkg.util as su
from sltxpkg.command_config import Arg, Commands
from sltxpkg.commands import (cmd_analyze_logfile, cmd_auto_setup, cmd_cleanse,
                              cmd_compile, cmd_dependency, cmd_docker,
                              cmd_gen_gha, cmd_raw_compile, cmd_version)


def valid_file(arg: str) -> str:
    if arg is None or arg.strip() == "":
        raise ValueError("arg vas none or empty")
    if not os.path.isfile(arg):
        raise FileNotFoundError("\"" + arg + "\" must be an existing file")
    return arg


file_help = "the file(s) to load; they will be processed."
file_tx = "file.tex"

sub_parser = Commands({
    'analyze': ((cmd_analyze_logfile, ['log']),
                Arg(description='Analyze a logfile for errors.'),
                [
                    Arg('files', metavar='log.zip',
                        type=str, nargs='*', help=file_help)]
                ),
    'dependency': ((cmd_dependency, ['dep']),
                   Arg(description='Install dependencies on the host system.'),
                   [
                       Arg('-l', '--local', metavar='path', dest='local_path', default=None,
                           help="This will install the dependency file into the given directory. This might be useful "
                                "if the compilation should be handled by an online editor. Use '.' to use the current "
                                "directory."),
                       Arg('deps', metavar='dep.yml', type=valid_file, nargs='+',
                           help="the file(s) to load the dependencies from.")
                   ]),
    'docker': ((cmd_docker, ['do']), Arg(description='Manage the containers to compile with sltx.'), []),
    'compile': ((cmd_compile, ['cmp']), Arg(
        description="Compile documents with previously installed containers. If docker was disabled this will default "
                    "to the same behavior as \"raw-compile\" passing on the recipe."),
                [
                    Arg('-p', '--profile', dest='profile',
                        help="allows to override the configured docker profile. This will enable docker automatically."),
                    Arg('-r', '--recipe', dest='recipe',
                        help='The recipe to instruct the main compile routine.', required=False, default=None),
                    Arg('--root', dest='dock_as_root', action='store_true',
                        help='Run the docker container with the lithie-root setup. This may lead to permission errors '
                             'when you want to delete the caches.',
                        required=False),
                    Arg('-l', '--local-texmf', dest='local_texmf', action='store_true', default=False,
                        help='include the local texmf tree in the docker container', required=False),
                    Arg('--no-local-texmf', dest='local_texmf', action='store_false',
                        help='exclude the local texmf tree in the docker container', required=False),
                    Arg('-d', '--dependencies', dest='extra_dependencies', action='append', default=[],
                        help='additional dependency files to download before the installation. May be supplied '
                             'multiple times.',
                        required=False),
                    Arg('-a', '--args', action='append', metavar='ARGUMENT(S)', dest='extra_arguments', default=[],
                        help="Extra arguments. Make sure to prepend them with the appropriate '-' or '--', they will "
                             "be used as extra_arguments in the recipe."),
                    Arg('files', metavar=file_tx, type=str, nargs='*',
                        help=file_help)
                ]),
    'raw-compile': ((cmd_raw_compile, ['raw-cmp']), Arg(
        description="Compile documents using a recipe. This will not start any docker container but will be executed "
                    "inside one as well."),
                    [
                        Arg('-r', '--recipe', dest='recipe',
                            help='The recipe to instruct the main compile routine.', required=False, default=None),
                        Arg('files', metavar=file_tx, type=str, nargs='*',
                            help=file_help),
                        Arg('-a', '--args', action='append', metavar='ARGUMENT(S)', dest='extra_arguments', default=[],
                            help="Extra arguments. Make sure to prepend them with the appropriate '-' or '--', "
                                 "they will be used as extra_arguments in the recipe."),
                        Arg('-d', '--dependency', dest='extra_dependencies', action='append', default=[],
                            help='additional dependency files to download before the installation. May be supplied '
                                 'multiple times.',
                            required=False)
                    ]),
    'gen-gha': ((cmd_gen_gha, ['gha']), Arg(description='Generate a GitHub workflow To automate compilation.'), []),
    'cleanse': ((cmd_cleanse, ['cls']), Arg(
        description="This will clean all additional sltx-files in the current directory (like \"sltx-log-*\" files). "
                    "It may clean more, if you pass the corresponding flags. Please note, that cleanse will only read "
                    "the current config. If you've changed some configurations they will be used."),
                [
                    Arg('-C', '--cache', dest='cleanse_cache', action='store_true',
                        help="If set, sltx will clean the cache."),
                    Arg('--all', dest='cleanse_all', action='store_true',
                        help="If set, sltx will clean the texmf-tree (sltx) and the cache as-well."),
                    Arg('-e', '--exclude', action='append', metavar='pattern', dest='exclude_patterns',
                        help="Exclude all files/directories matching this pattern. May be supplied multiple times.")
                ]),
    'auto-setup': (
    (cmd_auto_setup, []), Arg(description='Setup a basic version of sltx (this requires docker to be setup).'),
    [
        Arg('-d', '--dependencies', dest='auto_deps', action='store_true',
            help='This will install the recommended dependencies on your host system. This is helpful if you have '
                 'texlive installed and want your editor to recognize the libraries as well.')
    ]),
    'version': ((cmd_version, []), Arg(description='Show the version-info for sltx.'), [])
})

parser = argparse.ArgumentParser(
    description="sltx, a Simple LaTeX utility", epilog="sltx Version: " + su.get_version())

# commands, parser.add_mutually_exclusive_group()
parser.add_argument('-c', '--config', dest='config', metavar='config.yml',
                    required=False, type=valid_file,
                    help="the file to load the configuration from.")

parser.add_argument('-t', '--threads', metavar='N', dest='threads', type=int,
                    help="number of threads to run the installation. Default is 1. This number will only affect some "
                         "routines.",
                    default=-1)

parser.add_argument('-q', '--quiet', dest='quiet',
                    required=False, action='store_true',
                    help="Set the flag if output is to be reduced")

parser.add_argument('-v', '--verbose', dest='verbose',
                    required=False, action='store_true',
                    help="Output in verbose mode -> more information from sltx")

parser.add_argument('--log', dest='log',
                    required=False, action='store_true',
                    help="Write to logfile. This is experimental.")

# TODO: Format support with mlatexformat?
cmd_parser = parser.add_subparsers(title='command', description="Select the command for sltx", metavar=set(sub_parser.cmds.keys()),
    help="Help for the specific command. You may use shortcuts for them: " +
         str([k[1] for k in sub_parser.helper_values() if len(k[1]) != 0]),
    dest='command')

sub_parser.generate(cmd_parser)
